import threading
from typing import TypeVar, Generic, List,cast

from geniusweb.party.Party import Party
from geniusweb.connection.ConnectionEnd import ConnectionEnd
from geniusweb.inform.Inform import Inform
from geniusweb.actions.Action import Action

from tudelft.utilities.listener.Listener import Listener
from tudelft.utilities.logging.ReportToLogger import ReportToLogger
from tudelft.utilities.logging.Reporter import Reporter

INFORM = TypeVar('INFORM')

class DefaultParty (Party, Listener[INFORM]):
	'''
	Party with default implementation to handle the connection.
	'''

	def __init__(self, rep:Reporter=ReportToLogger("party")):
		self._reporter=rep
		self._connection: ConnectionEnd[Inform,Action] = cast(ConnectionEnd[Inform,Action],None)
		self._lock = threading.Lock()

	def connect(self, connection: ConnectionEnd[Inform, Action] ):
		self._connection = connection
		self._connection.addListener(self)

	#Override
	def disconnect(self):
		self._lock.acquire()
		if self._connection:
			self._connection.removeListener(self)
			self._connection.close()
			self._connection = None
		self._lock.release()
	

	#Override
	def terminate(self):
		self.disconnect();
	

	def getConnection(self) -> ConnectionEnd[Inform, Action]:
		'''
		@return currently available connection, or null if not currently
        connected.
		'''
		return self._connection;
	

	def getReporter(self)->Reporter:
		return self._reporter;

